@@ -1,58 +1,101 @@
 //! Work in progress game engine, inspired by [arcade](https://arcade.academy/).
 //!
 //! ```no_run
-//! use cat_box::{Event, Game, Keycode, Sprite};
+//! use cat_box::{draw_text, Game, Sprite, SpriteCollection, get_mouse_state, get_keyboard_state};
+//! use sdl2::keyboard::Scancode;
 //!
 //! fn main() {
-//!     let game = Game::new("cat_box demo", 1000, 800);
+//!    let game = Game::new("catbox demo", 1000, 800);
 //!
-//!     let mut i = 0;
+//!     let mut i = 0u8;
 //!     let mut s = Sprite::new("duck.png", 500, 400).unwrap();
-//!     game.run(|ctx, event_pump| {
+//!     let mut s2 = Sprite::new("duck.png", 400, 500).unwrap();
+//!
+//!     let mut coll = SpriteCollection::new();
+//!     for n in 0..10 {
+//!         for o in 0..8 {
+//!             let x = Sprite::new("duck.png", n * 100, o * 100).unwrap();
+//!             coll.push(x);
+//!         }
+//!     }
+//!     game.run(|ctx| {
 //!         i = (i + 1) % 255;
 //!         ctx.set_background_colour(i as u8, 64, 255);
 //!
+//!         draw_text(
+//!             ctx,
+//!             format!("i is {}", i),
+//!             "MesloLGS NF Regular.ttf",
+//!             72,
+//!             (300, 300),
+//!             cat_box::TextMode::Shaded {
+//!                 foreground: (255, 255, 255),
+//!                 background: (0, 0, 0),
+//!             },
+//!         )
+//!         .unwrap();
+//!
 //!         let (start_x, start_y) = s.position();
-//!         let m = sdl2::mouse::MouseState::new(event_pump.as_ref());
-//!         let x_diff = m.x() - start_x;
-//!         let y_diff = m.y() - start_y;
+//!         let m = get_mouse_state(ctx);
+//!         let x_diff = m.x - start_x;
+//!         let y_diff = m.y - start_y;
 //!
 //!         let angle = (y_diff as f64).atan2(x_diff as f64);
 //!         s.set_angle(angle.to_degrees());
 //!
-//!        for event in event_pump {
-//!             match event {
-//!                 Event::Quit { .. }
-//!                 | Event::KeyDown {
-//!                     keycode: Some(Keycode::Escape),
-//!                     ..
-//!                 } => game.terminate(),
+//!         for spr in coll.iter() {
+//!             let (start_x, start_y) = spr.position();
+//!             let m = get_mouse_state(ctx);
+//!             let x_diff = m.x - start_x;
+//!             let y_diff = m.y - start_y;
 //!
-//!                 Event::KeyDown { keycode, .. } => {
-//!                     let offset = match keycode.unwrap() {
-//!                         Keycode::W | Keycode::Up => (0, 5),
-//!                         Keycode::S | Keycode::Down => (0, -5),
-//!                         Keycode::A | Keycode::Left => (-5, 0),
-//!                         Keycode::D | Keycode::Right => (5, 0),
-//!                         _ => (0, 0),
-//!                     };
+//!             let angle = (y_diff as f64).atan2(x_diff as f64);
+//!             spr.set_angle(angle.to_degrees());
+//!         }
 //!
-//!                     s.translate(offset);
-//!                 }
-//!                 _ => {}
+//!         let keys = get_keyboard_state(ctx).keys;
+//!
+//!         for key in keys {
+//!             let offset = match key {
+//!                 Scancode::Escape => {
+//!                     game.terminate();
+//!                     (0, 0)
+//!                 },
+//!                 Scancode::W | Scancode::Up => (0, 5),
+//!                 Scancode::S | Scancode::Down => (0, -5),
+//!                 Scancode::A | Scancode::Left => (-5, 0),
+//!                 Scancode::D | Scancode::Right => (5, 0),
+//!                 _ => (0, 0),
+//!             };
+//!
+//!             s.translate(offset);
+//!
+//!             for spr in coll.iter() {
+//!                 spr.translate(offset);
 //!             }
 //!         }
 //!
+//!         s2.draw(ctx).unwrap();
 //!         s.draw(ctx).unwrap();
+//!         coll.draw(ctx).unwrap();
 //!     })
 //!     .unwrap();
 //! }
 //! ```
 
-use std::{cell::Cell, path::Path, ops::{DerefMut, Deref}, slice::IterMut};
+pub mod physics;
+
+use std::{
+    cell::Cell,
+    ops::{Deref, DerefMut},
+    path::Path,
+    slice::IterMut,
+};
 
 use sdl2::{
     image::ImageRWops,
+    keyboard::Scancode,
+    mouse::MouseButton,
     rect::Rect,
     render::{Canvas, TextureCreator, TextureValueError},
     rwops::RWops,
@@ -179,12 +222,12 @@ impl Sprite {
     /// # use cat_box::*;
     /// # let mut s = Sprite::new("duck.png", 500, 400).unwrap();
     /// # let game = Game::new("sprite demo", 1000, 1000);
-    /// # game.run(|ctx, _| {
+    /// # game.run(|ctx| {
     /// s.draw(ctx);
     /// # });
     /// ```
     pub fn draw(&mut self, ctx: &mut Context) -> Result<()> {
-        let (creator, canvas) = ctx.inner();
+        let (creator, canvas, _) = ctx.inner();
         let text = creator.create_texture_from_surface(&self.surf)?;
 
         canvas.copy_ex(&text, None, self.rect, self.angle, None, false, false)?;
@@ -241,8 +284,8 @@ impl Sprite {
     }
 }
 
-/// Manages a collection of [`Sprite`]s. 
-/// 
+/// Manages a collection of [`Sprite`]s.
+///
 /// Technically, this is a thin wrapper around a simple [`Vec`] of sprites,
 /// although with some convenience methods.
 pub struct SpriteCollection {
@@ -251,20 +294,18 @@ pub struct SpriteCollection {
 
 impl SpriteCollection {
     /// Creates a new [`SpriteCollection`].
-    /// 
+    ///
     /// See [`Vec::new()`] for more information.
     /// ```
     /// # use cat_box::*;
     /// let sprites = SpriteCollection::new();
     /// ```
     pub fn new() -> Self {
-        Self {
-            v: Vec::new()
-        }
+        Self { v: Vec::new() }
     }
 
     /// Creates a new [`SpriteCollection`] with the specified capacity.
-    /// 
+    ///
     /// The collection will be able to hold exactly `capacity` items without reallocating.
     /// ```
     /// # use cat_box::*;
@@ -272,7 +313,7 @@ impl SpriteCollection {
     /// ```
     pub fn with_capacity(cap: usize) -> Self {
         Self {
-            v: Vec::with_capacity(cap)
+            v: Vec::with_capacity(cap),
         }
     }
 
@@ -282,9 +323,9 @@ impl SpriteCollection {
     /// # use cat_box::*;
     /// # let mut sprites = SpriteCollection::new();
     /// # let mut game = Game::new("asjdfhalksjdf", 1, 1);
-    /// # game.run(|ctx, _| {
+    /// # game.run(|ctx| {
     /// sprites.draw(ctx);
-    /// # }); 
+    /// # });
     /// ```
     pub fn draw(&mut self, ctx: &mut Context) -> Result<()> {
         for s in self.v.iter_mut() {
@@ -293,7 +334,7 @@ impl SpriteCollection {
 
         Ok(())
     }
-    
+
     /// Add a new [`Sprite`] to the end of this collection.
     /// ```
     /// # use cat_box::*;
@@ -302,7 +343,7 @@ impl SpriteCollection {
     /// sprites.push(s);
     /// ```
     pub fn push(&mut self, s: Sprite) {
-        self.v.push(s); 
+        self.v.push(s);
     }
 
     /// Inserts an element at position `index` within the collection.
@@ -316,7 +357,7 @@ impl SpriteCollection {
     pub fn insert(&mut self, s: Sprite, index: usize) {
         self.v.insert(index, s);
     }
-    
+
     /// Removes and returns the last element, or `None` if the collection is empty.
     /// ```
     /// # use cat_box::*;
@@ -390,6 +431,11 @@ impl SpriteCollection {
     pub fn get(&self, index: usize) -> Option<&Sprite> {
         self.v.get(index)
     }
+
+    /// Return the inner Vec. Only use this method if you know what you're doing.
+    pub fn inner(&self) -> &Vec<Sprite> {
+        &self.v
+    }
 }
 
 impl Deref for SpriteCollection {
@@ -411,15 +457,17 @@ impl DerefMut for SpriteCollection {
 /// In most cases, this should never actually be used; instead, just pass it around to the various cat-box functions such as [`Sprite::draw()`].
 pub struct Context {
     canvas: Canvas<Window>,
+    event_pump: EventPump,
     texture_creator: TextureCreator<WindowContext>,
     ttf_subsystem: Sdl2TtfContext,
 }
 
 impl Context {
-    fn new(canvas: Canvas<Window>, ttf_subsystem: Sdl2TtfContext) -> Self {
+    fn new(canvas: Canvas<Window>, pump: EventPump, ttf_subsystem: Sdl2TtfContext) -> Self {
         let creator = canvas.texture_creator();
         Self {
             canvas,
+            event_pump: pump,
             texture_creator: creator,
             ttf_subsystem,
         }
@@ -428,8 +476,18 @@ impl Context {
     /// Get the inner [`Canvas`](sdl2::render::Canvas) and [`TextureCreator`](sdl2::render::TextureCreator).
     ///
     /// Only use this method if you know what you're doing.
-    pub fn inner(&mut self) -> (&TextureCreator<WindowContext>, &mut Canvas<Window>) {
-        (&self.texture_creator, &mut self.canvas)
+    pub fn inner(
+        &mut self,
+    ) -> (
+        &TextureCreator<WindowContext>,
+        &mut Canvas<Window>,
+        &mut EventPump,
+    ) {
+        (
+            &self.texture_creator,
+            &mut self.canvas,
+            &mut self.event_pump,
+        )
     }
 
     fn update(&mut self) {
@@ -440,6 +498,18 @@ impl Context {
         self.canvas.clear();
     }
 
+    fn check_for_quit(&mut self) -> bool {
+        let (_, _, pump) = self.inner();
+
+        for event in pump.poll_iter() {
+            if let Event::Quit { .. } = event {
+                return true;
+            }
+        }
+
+        false
+    }
+
     /// Set the background colour. See [`Canvas::set_draw_color()`](sdl2::render::Canvas::set_draw_color()) for more info.
     pub fn set_background_colour(&mut self, r: u8, g: u8, b: u8) {
         self.canvas.set_draw_color(Color::RGB(r, g, b));
@@ -470,7 +540,7 @@ pub enum TextMode {
 /// ``` no_run
 /// # use cat_box::*;
 /// # let game = Game::new("", 100, 100);
-/// # game.run(|ctx, _| {
+/// # game.run(|ctx| {
 /// let mode = TextMode::Shaded {
 ///     foreground: (255, 255, 255),
 ///     background: (0, 0, 0)
@@ -497,7 +567,7 @@ pub fn draw_text<S: AsRef<str>>(
     }?;
 
     drop(font);
-    let (creator, canvas) = ctx.inner();
+    let (creator, canvas, _) = ctx.inner();
     let texture = creator.create_texture_from_surface(&surf)?;
 
     let srect = surf.rect();
@@ -508,6 +578,58 @@ pub fn draw_text<S: AsRef<str>>(
     Ok(())
 }
 
+/// Representation of the mouse state.
+pub struct MouseRepr {
+    pub buttons: Vec<MouseButton>,
+    pub x: i32,
+    pub y: i32,
+}
+
+/// Representation of the keyboard state.
+pub struct KeyboardRepr {
+    pub keys: Vec<Scancode>,
+}
+
+/// Get the mouse state.
+/// ```no_run
+/// # use cat_box::*;
+/// # let game = Game::new("catbox-demo", 10, 10);
+/// # game.run(|ctx| {
+/// let m = get_mouse_state(ctx);
+/// println!("({}, {})", m.x, m.y);
+/// # });
+pub fn get_mouse_state(ctx: &mut Context) -> MouseRepr {
+    let (_, _, pump) = ctx.inner();
+
+    let mouse = pump.mouse_state();
+
+    MouseRepr {
+        buttons: mouse.pressed_mouse_buttons().collect(),
+        x: mouse.x(),
+        y: mouse.y(),
+    }
+}
+
+/// Get the keyboard state.
+/// ```no_run
+/// # use cat_box::*;
+/// # let game = Game::new("catbox-demo", 10, 10);
+/// # game.run(|ctx| {
+/// let k = get_keyboard_state(ctx);
+/// for code in k.keys {
+///     println!("{}", code);
+/// }
+/// # });
+pub fn get_keyboard_state(ctx: &mut Context) -> KeyboardRepr {
+    let (_, _, pump) = ctx.inner();
+
+    let keyboard = pump.keyboard_state();
+
+    KeyboardRepr {
+        keys: keyboard.pressed_scancodes().collect(),
+    }
+}
+
 /// Representation of the game.
 pub struct Game {
     /// The title that the window displays.
@@ -543,11 +665,11 @@ impl Game {
     /// ```no_run
     /// # use cat_box::Game;
     /// # let game = Game::new("Cool game", 1000, 1000);
-    /// game.run(|ctx, events| {
+    /// game.run(|ctx| {
     ///     // Game logic goes here
     /// });
     /// ```
-    pub fn run<F: FnMut(&mut Context, &mut Events)>(&self, mut func: F) -> Result<()> {
+    pub fn run<F: FnMut(&mut Context)>(&self, mut func: F) -> Result<()> {
         let sdl_context = sdl2::init()?;
         let video_subsystem = sdl_context.video()?;
 
@@ -563,16 +685,14 @@ impl Game {
 
         let event_pump = sdl_context.event_pump()?;
 
-        let mut events = Events { pump: event_pump };
-
-        let mut ctx = Context::new(canvas, s);
+        let mut ctx = Context::new(canvas, event_pump, s);
 
         loop {
-            if self.stopped.get() {
+            if self.stopped.get() || ctx.check_for_quit() {
                 break;
             }
             ctx.clear();
-            func(&mut ctx, &mut events);
+            func(&mut ctx);
             ctx.update();
         }
 
